@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -15,8 +15,11 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-#include <mobius/string_functions.h>
+#include "string_functions.h"
 #include <algorithm>
+#include <sstream>
+#include <iomanip>
+#include <fnmatch.h>
 
 namespace mobius
 {
@@ -29,18 +32,18 @@ namespace string
 //! \param s2 string
 //! \return new string
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 replace (const std::string& str, const std::string& s1, const std::string& s2)
 {
   if (str.empty () || s1.empty ())
     return std::string ();
 
   std::string tmp (str);
-  std::size_t pos = tmp.find (s1);
+  std::string::size_type pos = tmp.find (s1);
 
   while (pos != std::string::npos)
     {
-      tmp.replace (pos, s1.length (), s2);
+      tmp = tmp.replace (pos, s1.length (), s2);
       pos = tmp.find (s1, pos + s2.length ());
     }
 
@@ -52,14 +55,11 @@ replace (const std::string& str, const std::string& s1, const std::string& s2)
 //! \param str string
 //! \return lowercased string
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 tolower (const std::string& str)
 {
-  std::string tmp (str);
-
-  for (auto& c : tmp)
-    c = ::tolower (c);
-
+  std::string tmp (str.length (), 0);
+  std::transform (str.begin (), str.end (), tmp.begin (), ::tolower);
   return tmp;
 }
 
@@ -68,14 +68,11 @@ tolower (const std::string& str)
 //! \param str string
 //! \return uppercased string
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 toupper (const std::string& str)
 {
-  std::string tmp (str);
-
-  for (auto& c : tmp)
-    c = ::toupper (c);
-
+  std::string tmp (str.length (), 0);
+  std::transform (str.begin (), str.end (), tmp.begin (), ::toupper);
   return tmp;
 }
 
@@ -90,7 +87,7 @@ startswith (const std::string& str, const std::string& starting)
 {
   return starting.length () > 0 &&
          str.length () >= starting.length () &&
-         str.substr (0, starting.length ()) == starting;
+         str.compare (0, starting.length (), starting) == 0;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -104,7 +101,19 @@ endswith (const std::string& str, const std::string& ending)
 {
   return ending.length () > 0 &&
          str.length () >= ending.length () &&
-         str.substr (str.length () - ending.length (), ending.length ()) == ending;
+         str.compare (str.length () - ending.length (), ending.length (), ending) == 0;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief check if a string matches a given pattern
+//! \param patter fnmatch patter
+//! \param str string
+//! \return true if str matches, false otherwise
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+fnmatch (const std::string& pattern, const std::string& str)
+{
+  return ::fnmatch (pattern.c_str (), str.c_str (), FNM_NOESCAPE) == 0;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -113,7 +122,7 @@ endswith (const std::string& str, const std::string& ending)
 //! \param chars chars
 //! \return a new string
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 strip (const std::string& str, const std::string& chars)
 {
   const auto pos1 = str.find_first_not_of (chars);
@@ -121,7 +130,7 @@ strip (const std::string& str, const std::string& chars)
 
   if (pos1 == std::string::npos || pos2 == std::string::npos)
     return std::string ();
-  
+
   return str.substr (pos1, pos2 - pos1 + 1);
 }
 
@@ -131,7 +140,7 @@ strip (const std::string& str, const std::string& chars)
 //! \param chars chars
 //! \return a new string
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 lstrip (const std::string& str, const std::string& chars)
 {
   const auto pos = str.find_first_not_of (chars);
@@ -148,7 +157,7 @@ lstrip (const std::string& str, const std::string& chars)
 //! \param chars chars
 //! \return a new string
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 rstrip (const std::string& str, const std::string& chars)
 {
   const auto pos = str.find_last_not_of (chars);
@@ -165,13 +174,13 @@ rstrip (const std::string& str, const std::string& chars)
 //! \param sep separator string
 //! \return vector containing parts
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::vector <std::string>
+std::vector <std::string>
 split (const std::string& str, const std::string& sep)
 {
   std::vector <std::string> result;
   std::string::size_type start = 0;
   std::string::size_type end = str.find (sep);
-  
+
   while (end != std::string::npos)
     {
       result.push_back (str.substr (start, end - start));
@@ -183,13 +192,38 @@ split (const std::string& str, const std::string& sep)
   return result;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief get n word from string
+//! \param str string
+//! \param n word index (negative values count from right to left)
+//! \param sep separator string
+//! \return word
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::string
+word (const std::string& str, int n, const std::string& sep)
+{
+  std::string word;
+
+  // split string
+  auto d = split (str, sep);
+
+  if (n < 0)
+    n = d.size () + n;
+
+  // get n word
+  if (n >= 0 && n < int (d.size ()))
+    word = d[n];
+
+  return word;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief remove all occurrences of 'c' in str
 //! \param str string
 //! \param c char
 //! \return a new string
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 remove_char (const std::string& str, char c)
 {
   std::string tmp (str);
@@ -205,12 +239,12 @@ remove_char (const std::string& str, char c)
 //! \param c fill char
 //! \return a new string
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 lpad (const std::string& str, std::string::size_type siz, char c)
 {
   if (str.length () < siz)
     return std::string (siz - str.length (), c) + str;
-  
+
   return str;
 }
 
@@ -221,7 +255,7 @@ lpad (const std::string& str, std::string::size_type siz, char c)
 //! \param c fill char
 //! \return a new string
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 rpad (const std::string& str, std::string::size_type siz, char c)
 {
   if (str.length () < siz)
@@ -230,5 +264,32 @@ rpad (const std::string& str, std::string::size_type siz, char c)
   return str;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief convert value to hexadecimal string
+//! \param value value
+//! \param digits min digits
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::string
+to_hex (std::uint64_t value, unsigned int digits)
+{
+  std::ostringstream stream;
+  stream << std::hex << std::setw (digits) << std::setfill ('0') << value << std::dec;
+  return stream.str ();
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief convert value to string
+//! \param value value
+//! \param digits min digits
+//! \param fill fill char
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::string
+to_string (std::uint64_t value, unsigned int digits, char fill)
+{
+  std::ostringstream stream;
+  stream << std::setw (digits) << std::setfill (fill) << value;
+  return stream.str ();
+}
+
 } // namespace string
 } // namespace mobius
